// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package controller

import (
	"bytes"
	"encoding/json"
	"net/http"
	"testing"
	"time"

	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/adapter/controller/usecasemock"
	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/usecase/usecase"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/errno"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/test"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/token"
	"github.com/gin-gonic/gin"
	"github.com/stretchr/testify/assert"
)

func TestSessionCreate(t *testing.T) {
	tests := []struct {
		Name  string
		User  createParams
		Errno *errno.Errno
	}{
		{"registered user", createParams{
			Email:    "correct@correct.com",
			Password: "strong password",
		}, errno.OK},
		{"unregistered user", createParams{
			Email:    "unregistered@error.com",
			Password: "strong password",
		}, usecase.UserNotExist},
	}

	token.Init("secret", 1*time.Second)

	for _, tst := range tests {
		t.Run(tst.Name, func(t *testing.T) {
			testUserUsecase := usecasemock.NewtestUserUsecase()
			sessionController := NewSessionController(testUserUsecase)
			r := gin.New()
			r.POST(
				"/session/create",
				func(ctx *gin.Context) { sessionController.Create(ctx) },
			)
			user, _ := json.Marshal(tst.User)
			res := test.PerformRequest(t, r, "POST", "/session/create", bytes.NewReader(user),
				test.Header{
					Key:   "content-type",
					Value: "application/json",
				})

			assert.Equal(t, tst.Errno.HTTP, res.Result().StatusCode, res.Body)

			if tst.Errno.HTTP != http.StatusOK {
				var got errno.Errno
				err := json.NewDecoder(res.Result().Body).Decode(&got)
				// XXX: the http status is not in the json. So it must be reset back the the struct
				got.HTTP = res.Result().StatusCode
				assert.NoError(t, err)
				assert.Equal(t, tst.Errno, &got)
				return
			}

			var got Token
			err := json.NewDecoder(res.Result().Body).Decode(&got)
			assert.NoError(t, err)
			tkResp, err := token.Parse(got.Token)
			assert.NoError(t, err)
			assert.Equal(t, tst.User.Email, tkResp.Identity)
		})
	}
}
